package dev.fyloz.colorrecipesexplorer.repository

import dev.fyloz.colorrecipesexplorer.model.Material
import org.springframework.data.jpa.repository.JpaRepository
import org.springframework.data.jpa.repository.Modifying
import org.springframework.data.jpa.repository.Query
import org.springframework.stereotype.Repository

@Repository
interface MaterialRepository : JpaRepository<Material, Long> {
    /** Checks if a material with the given [name] and a different [id] exists. */
    fun existsByNameAndIdNot(name: String, id: Long): Boolean

    /** Gets all non mix type materials. */
    fun getAllByIsMixTypeIsFalse(): Collection<Material>

    /** Updates the [inventoryQuantity] of the [Material] with the given [id]. */
    @Modifying
    @Query("UPDATE Material m SET m.inventoryQuantity = :inventoryQuantity WHERE m.id = :id")
    fun updateInventoryQuantityById(id: Long, inventoryQuantity: Float)

    /** Checks if a mix material or a mix type depends on the material with the given [id]. */
    @Query(
        """
        select case when(count(mm.id) + count(mt.id) > 0) then false else true end
        from Material m
        left join MixMaterial mm on mm.material.id = m.id
        left join MixType mt on mt.material.id = m.id
        where m.id = :id
        """
    )
    fun isUsedByMixMaterialOrMixType(id: Long): Boolean
}
